var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

var userId = null;

describe('CreateUser', function () {
  
    it('should create a user with required fields (userName, lastName, directoryId)', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'createUser',
        'args': {
          'input': {
            'userName': 'waylon_dalton',
            'lastName' : 'Dalton',
            'directoryId': 'RGlyZWN0b3J5OjI='
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    });

    it('should return the user created with the \'waylon_dalton\' username ', function (done) {
      if (userId === null) {
        this.skip();
      }
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateUserProperties(res.body);

        expect(res.body).to.include({ id: userId, userName: 'waylon_dalton', lastName: "Dalton", directoryId: "RGlyZWN0b3J5OjI=" });
        
        done();
      });
    });

    it('should create a user with some pertinent fields', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'createUser',
        'args': {
          'input': {
            'userName': 'tory_anan',
            'lastName': 'Anan',
            'firstName': 'Tory',
            'directoryId': 'RGlyZWN0b3J5OjI=',
            'isAdministrator': true,
            'isActive': true,
            'email': 'tory_anan@advantys.com',
            'managerId': 'VXNlcjox'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    }); 

    it('should return the user created with the \'tory_anan\' username ', function (done) {
      if (userId === null) {
        this.skip();
      }
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateUserProperties(res.body);

        expect(res.body).to.include({ id: userId, userName: 'tory_anan', lastName: "Anan", firstName: 'Tory', directoryId: "RGlyZWN0b3J5OjI=",
                                      isAdministrator: true, isActive: true, email: 'tory_anan@advantys.com', managerId: 'VXNlcjox'});        
        done();
      });
    });

    it('should create a user with a password (managed by WorkflowGen)', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'createUser',
        'args': {
          'input': {
            'userName': 'loukas_davor',
            'lastName': 'Davor',
            'directoryId': 'RGlyZWN0b3J5OjM=',
            'password': 'TestPassword$',
            'isAdministrator': true,
            'isActive': true
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    }); 

    it('should return the user created with the \'loukas_davor\' username ', function (done) {
      if (userId === null) {
        this.skip();
      }
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateUserProperties(res.body);

        expect(res.body).to.include({ id: userId, userName: 'loukas_davor', lastName: "Davor", directoryId: "RGlyZWN0b3J5OjM=",
                                      isAdministrator: true, isActive: true });        
        done();
      });
    });

    it('should get an error when the username \'waylon_dalton\' is already in use', function (done) {
      chai.request(url_wfgen_admin)
        .post('/')
        .send({
          'operation': 'createUser',
          'args': {
            'input': {
              'userName': 'waylon_dalton',
              'lastName' : 'Dalton',
              'directoryId': 'RGlyZWN0b3J5OjI='
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(500);
          res.should.be.json;

          res.body.should.have.property('error');
          expect(res.body.error).to.contain('The username is already in use');

          done();
        });
    });

    it('should get an error when the email is not valid', function (done) {
      chai.request(url_wfgen_admin)
        .post('/')
        .send({
          'operation': 'createUser',
          'args': {
            'input': {
              'userName': 'shelly_anita',
              'lastName' : 'Shelly',
              'directoryId': 'RGlyZWN0b3J5OjI=',
              'email': 'shelly_anita{}advantys.com'
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(500);
          res.should.be.json;

          res.body.should.have.property('error');
          expect(res.body.error).to.contain('Invalid email');

          done();
        });
    });

    it('should get an error when the password is not set and is managed by WorkflowGen', function (done) {
      chai.request(url_wfgen_admin)
        .post('/')
        .send({
          'operation': 'createUser',
          'args': {
            'input': {
              'userName': 'saul_martine',
              'lastName' : 'Shelly',
              'directoryId': 'RGlyZWN0b3J5OjM='
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(500);
          res.should.be.json;

          res.body.should.have.property('error');
          expect(res.body.error).to.contain('The password is required');

          done();
        });
    });
});
