var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
var fileUtils = require('../../sql/utils');
const config = require('../../../config.test');
chai.use(chaiHttp);
chai.use(chaiThings);

var requestId = null;
var requestNumber = 0;
var actionId = null;
var filesCount = 0;
var counter = 1;
var isNotificationCreated = false;

var applicationSmtpPickupDirectory = config.test.applicationSmtpPickupDirectory;

describe('CreateRequest-INITIATES-VALIDATES', function () {
  before(function () {
    var chaiFs = require('chai-fs');
    chai.use(chaiFs);

    fileUtils.countFilesInFolder(applicationSmtpPickupDirectory)
      .then((filesLength) => {
        filesCount = filesLength;
      });
  });

  it('should create a TEST_SIMPLE_APPROVAL new request', function (done) {
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'createRequest',
        'args': {
          'input': {
            'processName': 'TEST_SIMPLE_APPROVAL',
            'processVersion': 1
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        requestId = res.body.requestId;
        requestNumber = res.body.number;
        done();
      });
  });
  it('should return the request created', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestProperties(res.body);
        expect(res.body).to.include({ id: requestId, description: 'Simple approval', status: 'OPEN',
        isTest: true, processId: 'UHJvY2Vzczox', requesterId: 'VXNlcjo4' });
        done();
      });
  });
  it('should check that the New action to do notification for the action INITIATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter += 2;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action INITIATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  1-The Requester initiates');
        expect(mail.to.value[0].name).to.contain('User Requester');
        isNotificationCreated = false;
      });
  });
  it('should complete the action INITIATES', function (done) {
    if (requestNumber === null) {
      this.skip();
    }
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 1,
            'parameters': [{
              'name': 'REQUEST_SUBJECT',
              'textValue': 'Test subject'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'INITIATES-1.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'INITIATES-1.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'INITIATES-1.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'INITIATES-1.html'
                }
              }
            ]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });
  it('should return the action completed INITIATES', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '1-INITIATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action INITIATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 5);
        expect(res.body.items[0]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[0].fileValue).to.include({name: 'INITIATES-1.html', description: 'INITIATES-1.html', contentType: 'text/html', size: fileUtils.getFileSize('INITIATES-1.html')});
        expect(res.body.items[1]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'INITIATES-1.xml', description: 'INITIATES-1.xml', contentType: 'text/xml', size: fileUtils.getFileSize('INITIATES-1.xml')});
        expect(res.body.items[4]).to.include({name: 'REQUEST_SUBJECT', type: 'TEXT', textValue: 'Test subject'});
        done();
      });
  });
  it('should check that the New action to do notification for the action VALIDATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter++;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action VALIDATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  2-The Manager validates');
        expect(mail.to.value[0].name).to.contain('User Manager');
        isNotificationCreated = false;
      });
  });
  it('should complete the action VALIDATES with a Yes answer', function (done) {
    if (requestNumber === null) {
      this.skip();
    }
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 2,
            'parameters': [{
              'name': 'APPROVAL_DECISION',
              'textValue': 'Yes'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'VALIDATES-1.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'VALIDATES-1.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'VALIDATES-1.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'VALIDATES-1.html'
                }
              }]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });

  it('should return the action VALIDATES completed  with a Yes answer', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '2-VALIDATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action VALIDATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 4);
        expect(res.body.items[0]).to.include({name: 'APPROVAL_DECISION', type: 'TEXT', textValue: 'Yes'});
        expect(res.body.items[1]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'VALIDATES-1.html', description: 'VALIDATES-1.html', contentType: 'text/html', size: fileUtils.getFileSize('VALIDATES-1.html')});
        expect(res.body.items[2]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[2].fileValue).to.include({name: 'VALIDATES-1.xml', description: 'VALIDATES-1.xml', contentType: 'text/xml', size: fileUtils.getFileSize('VALIDATES-1.xml')});
        done();
      });
  });

  it('should return the request completed', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestProperties(res.body);
        expect(res.body).to.include({ id: requestId, description: 'Simple approval', status: 'CLOSED',  subStatus: 'COMPLETED',
        isTest: true, processId: 'UHJvY2Vzczox', requesterId: 'VXNlcjo4' });
        done();
      });
  });
  it('should check that the Request closed notification has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter++;
    isNotificationCreated = true;
    done();
  });
  it('should get the Request closed notification', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('Request closed');
        expect(mail.to.value[0].name).to.contain('User Requester');
        isNotificationCreated = false;
      });
  });
  it('should get the request dataset', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequestDataset',
        'args': {
          'requestId': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestDataPageListProperties(res.body, 11);
        expect(res.body.items[0]).to.include({name: 'APPROVAL_DECISION', type: 'TEXT', textValue: 'Yes'});
        expect(res.body.items[3]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[3].fileValue).to.include({name: 'VALIDATES-1.html', description: 'VALIDATES-1.html', contentType: 'text/html', size: fileUtils.getFileSize('VALIDATES-1.html')});
        expect(res.body.items[4]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[4].fileValue).to.include({name: 'VALIDATES-1.xml', description: 'VALIDATES-1.xml', contentType: 'text/xml', size: fileUtils.getFileSize('VALIDATES-1.xml')});
        expect(res.body.items[9]).to.include({name: 'REQUEST_SUBJECT', type: 'TEXT', textValue: 'Test subject'});
        done();
      });
  });
});

describe('CreateRequest-INITIATES-VALIDATES-UPDATES-VALIDATES', function () {
  before(function () {
    requestId = null;
    requestNumber = 0;
    actionId = null;
  });
  it('should create a TEST_SIMPLE_APPROVAL new request', function (done) {
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'createRequest',
        'args': {
          'input': {
            'processName': 'TEST_SIMPLE_APPROVAL',
            'processVersion': 1
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        requestId = res.body.requestId;
        requestNumber = res.body.number;
        done();
      });
  });
  it('should return the request created', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestProperties(res.body);
        expect(res.body).to.include({ id: requestId, description: 'Simple approval', status: 'OPEN',
        isTest: true, processId: 'UHJvY2Vzczox', requesterId: 'VXNlcjo4' });
        done();
      });
  });
  it('should check that the New action to do notification for the action INITIATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter += 2;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action INITIATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  1-The Requester initiates');
        expect(mail.to.value[0].name).to.contain('User Requester');
        isNotificationCreated = false;
      });
  });
  it('should complete the action INITIATES', function (done) {
    if (requestNumber === null) {
      this.skip();
    }
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 1,
            'parameters': [{
              'name': 'REQUEST_SUBJECT',
              'textValue': 'Second test subject'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'INITIATES-2.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'INITIATES-2.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'INITIATES-2.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'INITIATES-2.html'
                }
              }]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });
  it('should return the action INITIATES completed', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '1-INITIATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action INITIATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 5);
        expect(res.body.items[0]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[0].fileValue).to.include({name: 'INITIATES-2.html', description: 'INITIATES-2.html', contentType: 'text/html', size: fileUtils.getFileSize('INITIATES-2.html')});
        expect(res.body.items[1]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'INITIATES-2.xml', description: 'INITIATES-2.xml', contentType: 'text/xml', size: fileUtils.getFileSize('INITIATES-2.xml')});
        expect(res.body.items[4]).to.include({name: 'REQUEST_SUBJECT', type: 'TEXT', textValue: 'Second test subject'});
        done();
      });
  });
  it('should check that the New action to do notification for the action VALIDATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter++;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action VALIDATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  2-The Manager validates');
        expect(mail.to.value[0].name).to.contain('User Manager');
        isNotificationCreated = false;
      });
  });
  it('should complete the action VALIDATES with a More info answer', function (done) {
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 2,
            'parameters': [{
              'name': 'APPROVAL_DECISION',
              'textValue': 'MORE_INFO'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'VALIDATES-2-1.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'VALIDATES-2-1.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'VALIDATES-2-1.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'VALIDATES-2-1.html'
                }
              }]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });

  it('should return the action VALIDATES completed  with a More info answer', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '2-VALIDATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action VALIDATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 4);
        expect(res.body.items[0]).to.include({name: 'APPROVAL_DECISION', type: 'TEXT', textValue: 'MORE_INFO'});
        expect(res.body.items[1]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'VALIDATES-2-1.html', description: 'VALIDATES-2-1.html', contentType: 'text/html', size: fileUtils.getFileSize('VALIDATES-2-1.html')});
        expect(res.body.items[2]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[2].fileValue).to.include({name: 'VALIDATES-2-1.xml', description: 'VALIDATES-2-1.xml', contentType: 'text/xml', size: fileUtils.getFileSize('VALIDATES-2-1.xml')});
        done();
      });
  });
  it('should check that the New action to do notification for the action UPDATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter += 2;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action UPDATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  3-The requester updates');
        expect(mail.to.value[0].name).to.contain('User Requester');
        isNotificationCreated = false;
      });
  });
  it('should complete the action UPDATES', function (done) {
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 3,
            'parameters': [{
              'name': 'REQUEST_SUBJECT',
              'textValue': 'Updated test subject'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'UPDATES-2.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'UPDATES-2.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'UPDATES-2.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'UPDATES-2.html'
                }
              }]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });

  it('should return the action UPDATES completed', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '3-UPDATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action UPDATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 5);
        expect(res.body.items[0]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[0].fileValue).to.include({name: 'UPDATES-2.html', description: 'UPDATES-2.html', contentType: 'text/html', size: fileUtils.getFileSize('UPDATES-2.html')});
        expect(res.body.items[1]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'UPDATES-2.xml', description: 'UPDATES-2.xml', contentType: 'text/xml', size: fileUtils.getFileSize('UPDATES-2.xml')});
        expect(res.body.items[4]).to.include({name: 'REQUEST_SUBJECT', type: 'TEXT', textValue: 'Updated test subject'});
        done();
      });
  });

  it('should check that the New action to do notification for the action VALIDATES has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    counter++;
    isNotificationCreated = true;
    done();
  });
  it('should get the New action to do notification for the action VALIDATES', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('New action to do:  4-The Manager validates');
        expect(mail.to.value[0].name).to.contain('User Manager');
        isNotificationCreated = false;
      });
  });
  it('should complete the action VALIDATES with a No answer', function (done) {
    if (requestNumber === null) {
      this.skip();
    }
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'completeAction',
        'args': {
          'input': {
            'requestNumber': requestNumber,
            'number': 4,
            'parameters': [{
              'name': 'APPROVAL_DECISION',
              'textValue': 'No'
            },
              {
                'name': 'FORM_DATA',
                'fileValue': {
                  'name': 'VALIDATES-2-2.xml',
                  'contentType': 'text/xml',
                  'url': processFilesUrl + 'VALIDATES-2-2.xml'
                }
              },
              {
                'name': 'FORM_ARCHIVE',
                'fileValue': {
                  'name': 'VALIDATES-2-2.html',
                  'contentType': 'text/html',
                  'url': processFilesUrl + 'VALIDATES-2-2.html'
                }
              }]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        actionId = res.body.actionId;
        done();
      });
  });

  it('should return the action VALIDATES completed with a No answer', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionProperties(res.body);
        expect(res.body).to.include({ id: actionId, name: '4-VALIDATES', status: 'CLOSED', subStatus: 'COMPLETED' });
        done();
      });
  });
  it('should get the action VALIDATES dataset', function (done) {
    if (actionId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getActionDataset',
        'args': {
          'actionId': actionId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateActionDataPageListProperties(res.body, 4);
        expect(res.body.items[0]).to.include({name: 'APPROVAL_DECISION', type: 'TEXT', textValue: 'No'});
        expect(res.body.items[1]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[1].fileValue).to.include({name: 'VALIDATES-2-2.html', description: 'VALIDATES-2-2.html', contentType: 'text/html', size: fileUtils.getFileSize('VALIDATES-2-2.html')});
        expect(res.body.items[2]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[2].fileValue).to.include({name: 'VALIDATES-2-2.xml', description: 'VALIDATES-2-2.xml', contentType: 'text/xml', size: fileUtils.getFileSize('VALIDATES-2-2.xml')});
        done();
      });
  });
  it('should return the request completed', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestProperties(res.body);
        expect(res.body).to.include({ id: requestId, description: 'Simple approval', status: 'CLOSED', subStatus: 'COMPLETED',
        isTest: true, processId: 'UHJvY2Vzczox', requesterId: 'VXNlcjo4' });
        done();
      });
  });
  it('should check that the Request closed notification has been created', function (done) {
    expect(applicationSmtpPickupDirectory).to.be.a.directory().and.files.have.lengthOf(filesCount + counter);
    isNotificationCreated = true;
    done();
  });
  it('should get the Request closed notification', function () {
    if (!isNotificationCreated) {
      this.skip();
    }
    return fileUtils.getNewestEmailContent(applicationSmtpPickupDirectory)
      .then((mail) => {
        expect(mail.subject).to.contain('Request closed');
        expect(mail.to.value[0].name).to.contain('User Requester');
      });
  });
  it('should get the request dataset', function (done) {
    if (requestId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequestDataset',
        'args': {
          'requestId': requestId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        utils.validateRequestDataPageListProperties(res.body, 11);
        expect(res.body.items[0]).to.include({name: 'APPROVAL_DECISION', type: 'TEXT', textValue: 'No'});
        expect(res.body.items[3]).to.include({name: 'FORM_ARCHIVE', type: 'FILE'});
        expect(res.body.items[3].fileValue).to.include({name: 'VALIDATES-2-2.html', description: 'VALIDATES-2-2.html', contentType: 'text/html', size: fileUtils.getFileSize('VALIDATES-2-2.html')});
        expect(res.body.items[4]).to.include({name: 'FORM_DATA', type: 'FILE'});
        expect(res.body.items[4].fileValue).to.include({name: 'VALIDATES-2-2.xml', description: 'VALIDATES-2-2.xml', contentType: 'text/xml', size: fileUtils.getFileSize('VALIDATES-2-2.xml')});
        expect(res.body.items[9]).to.include({name: 'REQUEST_SUBJECT', type: 'TEXT', textValue: 'Updated test subject'});
        done();
      });
  });
});
